@@ -17,6 +17,11 @@ use build::probe;
 sub defined_or($$) {
     defined $_[0] ? $_[0] : $_[1]
 }
+# This allows us to run on perls without List::Util
+sub uniq {
+    my %s; grep { !$s{$_}++ } @_;
+}
+
 
 my $NAME    = 'moar';
 my $GENLIST = 'build/gen.list';
@@ -28,7 +33,9 @@ my $failed = 0;
 my %args;
 my %defaults;
 my %config;
-
+# In case a submodule folder needs to be deleted. We set this and print it
+# out at the very end.
+my $folder_to_delete = '';
 my @args = @ARGV;
 
 GetOptions(\%args, qw(
@@ -39,7 +46,7 @@ GetOptions(\%args, qw(
     static has-libtommath has-libatomic_ops
     has-dyncall has-libffi pkgconfig=s
     build=s host=s big-endian jit! enable-jit
-    prefix=s bindir=s libdir=s mastdir=s make-install asan ubsan valgrind telemeh),
+    prefix=s bindir=s libdir=s mastdir=s make-install asan ubsan valgrind telemeh show-autovect show-autovect-failed:s),
 
     'no-optimize|nooptimize' => sub { $args{optimize} = 0 },
     'no-debug|nodebug' => sub { $args{debug} = 0 },
@@ -71,7 +78,13 @@ if (-d '.git') {
     print dots("Updating submodules");
     my $msg = qx{git submodule sync --quiet && git submodule --quiet update --init 2>&1};
     if ($? >> 8 == 0) { print "OK\n" }
-    else { softfail("git error: $msg") }
+    else {
+        if ($msg =~ /[']([^']+)[']\s+already exists and is not an empty/) {
+            $folder_to_delete = "\n\nERROR: Cannot update submodule because directory exists and is not empty.\n" .
+            ">>> Please delete the following folder and try again:\n$1\n\n";
+        }
+        softfail("git error: $msg")
+    }
 }
 
 # fiddle with flags
@@ -80,7 +93,7 @@ $args{debug}        = 3 if defined $args{debug} and $args{debug} eq "";
 
 
 for (qw(coverage instrument static big-endian has-libtommath has-sha has-libuv
-        has-libatomic_ops asan ubsan valgrind)) {
+        has-libatomic_ops asan ubsan valgrind show-vec)) {
     $args{$_} = 0 unless defined $args{$_};
 }
 
@@ -142,6 +155,7 @@ $config{dllimport}                = '' unless defined $config{dllimport};
 $config{dllexport}                = '' unless defined $config{dllexport};
 $config{dlllocal}                 = '' unless defined $config{dlllocal};
 $config{translate_newline_output} = 0  unless defined $config{translate_newline_output};
+$config{vectorizerspecifier}      = '' unless defined $config{vectorizerspecifier};
 
 # assume the compiler can be used as linker frontend
 $config{ld}           = $config{cc} unless defined $config{ld};
@@ -151,6 +165,8 @@ $config{ldoptiflags}  = $config{ccoptiflags} unless defined $config{ldoptiflags}
 $config{lddebugflags} = $config{ccdebugflags} unless defined $config{lddebugflags};
 $config{ldinstflags}  = $config{ccinstflags} unless defined $config{ldinstflags};
 
+$config{as}           = $config{cc} unless defined $config{as};
+
 # Probe the compiler.
 build::probe::compiler_usability(\%config, \%defaults);
 
@@ -233,7 +249,7 @@ if ($args{'has-libatomic_ops'}) {
     }
 }
 else {
-    $config{cincludes} .= ' ' . $defaults{ccinc} . '3rdparty/libatomic_ops/src';
+    $config{cincludes} .= ' ' . $defaults{ccinc} . '3rdparty/libatomicops/src';
     my $lao             = '$(DESTDIR)$(PREFIX)/include/libatomic_ops';
     $config{install}   .= "\t\$(MKPATH) $lao/atomic_ops/sysdeps/armcc\n"
                         . "\t\$(MKPATH) $lao/atomic_ops/sysdeps/gcc\n"
@@ -243,17 +259,17 @@ else {
                         . "\t\$(MKPATH) $lao/atomic_ops/sysdeps/loadstore\n"
                         . "\t\$(MKPATH) $lao/atomic_ops/sysdeps/msftc\n"
                         . "\t\$(MKPATH) $lao/atomic_ops/sysdeps/sunc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/*.h $lao\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/*.h $lao/atomic_ops\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/*.h $lao/atomic_ops/sysdeps\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/armcc/*.h $lao/atomic_ops/sysdeps/armcc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/gcc/*.h $lao/atomic_ops/sysdeps/gcc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/hpc/*.h $lao/atomic_ops/sysdeps/hpc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/ibmc/*.h $lao/atomic_ops/sysdeps/ibmc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/icc/*.h $lao/atomic_ops/sysdeps/icc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/loadstore/*.h $lao/atomic_ops/sysdeps/loadstore\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/msftc/*.h $lao/atomic_ops/sysdeps/msftc\n"
-                        . "\t\$(CP) 3rdparty/libatomic_ops/src/atomic_ops/sysdeps/sunc/*.h $lao/atomic_ops/sysdeps/sunc\n";
+                        . "\t\$(CP) 3rdparty/libatomicops/src/*.h $lao\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/*.h $lao/atomic_ops\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/*.h $lao/atomic_ops/sysdeps\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/armcc/*.h $lao/atomic_ops/sysdeps/armcc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/gcc/*.h $lao/atomic_ops/sysdeps/gcc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/hpc/*.h $lao/atomic_ops/sysdeps/hpc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/ibmc/*.h $lao/atomic_ops/sysdeps/ibmc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/icc/*.h $lao/atomic_ops/sysdeps/icc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/loadstore/*.h $lao/atomic_ops/sysdeps/loadstore\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/msftc/*.h $lao/atomic_ops/sysdeps/msftc\n"
+                        . "\t\$(CP) 3rdparty/libatomicops/src/atomic_ops/sysdeps/sunc/*.h $lao/atomic_ops/sysdeps/sunc\n";
 }
 
 if ($args{'has-libtommath'}) {
@@ -326,7 +342,7 @@ else {
 $config{ldlibs} = join ' ',
     (map { sprintf $config{ldusr}, $_; } @{$config{usrlibs}}),
     (map { sprintf $config{ldsys}, $_; } @{$config{syslibs}});
-$config{ldlibs} = ' -lasan ' . $config{ldlibs} if $args{asan} and $^O ne 'darwin';
+$config{ldlibs} = ' -lasan ' . $config{ldlibs} if $args{asan} && $^O ne 'darwin' && $config{cc} ne 'clang';
 $config{ldlibs} = ' -lubsan ' . $config{ldlibs} if $args{ubsan} and $^O ne 'darwin';
 # macro defs
 $config{ccdefflags} = join ' ', map { $config{ccdef} . $_ } @{$config{defs}};
@@ -347,6 +363,23 @@ push @cflags, $config{ld_covflags}  if $args{coverage};
 push @cflags, $config{ccwarnflags};
 push @cflags, $config{ccdefflags};
 push @cflags, $config{ccshared}     unless $args{static};
+push @cflags,
+$config{cc} eq 'clang'
+    ? '-Rpass=loop-vectorize'
+: $config{cc} eq 'gcc'
+    ? '-fopt-info-vec-optimized'
+    : die if $args{'show-autovect'};
+push @cflags, $config{ccjitflags} if $args{jit};
+
+if (exists $args{'show-autovect-failed'}) {
+    push @cflags, '-Rpass-missed=loop-vectorize' if $config{cc} eq 'clang';
+    push @cflags, ("-ftree-vectorizer-verbose=" . ($args{'show-autovect-failed'} || 1), "-fopt-info-vec-missed")
+        if $config{cc} eq 'gcc';
+}
+if ($args{'show-autovect-failed'}) {
+    push @cflags, '-Rpass-analysis=loop-vectorize' if 2 <= $args{'show-autovect-failed'} && $config{cc} eq 'clang';
+    push @cflags, '-fsave-optimization-record '    if 3 <= $args{'show-autovect-failed'} && $config{cc} eq 'clang';
+}
 push @cflags, '-fno-omit-frame-pointer' if $args{asan} or $args{ubsan};
 push @cflags, '-fsanitize=address' if $args{asan};
 push @cflags, '-fsanitize=undefined' if $args{ubsan};
@@ -357,7 +390,7 @@ push @cflags, '-DHAVE_TELEMEH' if $args{telemeh};
 push @cflags, '-DWORDS_BIGENDIAN' if $config{be}; # 3rdparty/sha1 needs it and it isnt set on mips;
 push @cflags, $ENV{CFLAGS} if $ENV{CFLAGS};
 push @cflags, $ENV{CPPFLAGS} if $ENV{CPPFLAGS};
-$config{cflags} = join ' ', @cflags;
+$config{cflags} = join ' ', uniq(@cflags);
 
 # generate LDFLAGS
 my @ldflags = ($config{ldmiscflags});
@@ -583,7 +616,7 @@ TERM2
 if (!$failed && $args{'make-install'}) {
     system($config{make}, 'install');
 }
-
+print $folder_to_delete if $folder_to_delete;
 exit $failed;
 
 # helper functions
@@ -907,6 +940,19 @@ options.
 Explicitly set the compiler without affecting other configuration
 options.
 
+=item --show-autovect
+
+Prints debug messages when compiling showing which loops were auto vectorized
+to SIMD instructions during build. Option is supported for Clang and GCC only.
+
+=item --show-autovect-failed
+
+Prints debug messages which hopefully reveal why autovectorization has failed
+for a loop. Verbosity level is 1-3 for clang, for GCC it is likely 1-2.
+If you are trying to vectorize code, it's *highly* recommended to try using clang
+first as it's smarter and has more useful messages. Then once it is working,
+try to get it working on gcc.
+
 =item --asan
 
 Build with AddressSanitizer (ASAN) support. Requires clang and LLVM 3.1 or newer.
