@@ -203,15 +203,15 @@ static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
 }
 
 /* Helper for complaining about attribute access errors. */
-MVM_NO_RETURN
-static void no_such_attribute(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, MVMSTable *target_type) {
+MVM_NO_RETURN static void no_such_attribute(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, MVMSTable *target_type) MVM_NO_RETURN_ATTRIBUTE;
+MVM_NO_RETURN static void no_such_attribute(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, MVMSTable *target_type) {
     char *c_name = MVM_string_utf8_encode_C_string(tc, name);
     char *waste[] = { c_name, NULL };
     MVM_exception_throw_adhoc_free(tc, waste, "P6opaque: no such attribute '%s' on type %s in a %s when trying to %s", c_name, MVM_6model_get_debug_name(tc, class_handle), MVM_6model_get_stable_debug_name(tc, target_type), action);
 }
 
-MVM_NO_RETURN
-static void invalid_access_kind(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, const char *kind_desc) {
+MVM_NO_RETURN static void invalid_access_kind(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, const char *kind_desc) MVM_NO_RETURN_ATTRIBUTE;
+MVM_NO_RETURN static void invalid_access_kind(MVMThreadContext *tc, const char *action, MVMObject *class_handle, MVMString *name, const char *kind_desc) {
     char *c_name = MVM_string_utf8_encode_C_string(tc, name);
     char *waste[] = { c_name, NULL };
     MVM_exception_throw_adhoc_free(tc, waste, "P6opaque: invalid %s attribute '%s' in type %s for kind %s", action, c_name, MVM_6model_get_debug_name(tc, class_handle), kind_desc);
@@ -431,7 +431,8 @@ static MVMint64 hint_for(MVMThreadContext *tc, MVMSTable *st, MVMObject *class_k
 /* Gets an architecture atomic sized native integer attribute as an atomic
  * reference. */
 static AO_t * attribute_as_atomic(MVMThreadContext *tc, MVMSTable *st, void *data,
-                                  MVMObject *class_handle, MVMString *name) {
+                                  MVMObject *class_handle, MVMString *name,
+                                  MVMuint16 kind) {
     MVMP6opaqueREPRData *repr_data = (MVMP6opaqueREPRData *)st->REPR_data;
     MVMint64 slot;
     data = MVM_p6opaque_real_data(tc, data);
@@ -440,15 +441,24 @@ static AO_t * attribute_as_atomic(MVMThreadContext *tc, MVMSTable *st, void *dat
             "P6opaque: must compose %s before using get_attribute", MVM_6model_get_stable_debug_name(tc, st));
     slot = try_get_slot(tc, repr_data, class_handle, name);
     if (slot >= 0) {
-        MVMSTable *attr_st = repr_data->flattened_stables[slot];
-        if (attr_st) {
-            const MVMStorageSpec *ss = attr_st->REPR->get_storage_spec(tc, attr_st);
-            if (ss->inlineable && ss->boxed_primitive == MVM_STORAGE_SPEC_BP_INT &&
-                    ss->bits / 8 == sizeof(AO_t))
-                return (AO_t *)((char *)data + repr_data->attribute_offsets[slot]);
+        if (kind == MVM_reg_int64) {
+            MVMSTable *attr_st = repr_data->flattened_stables[slot];
+            if (attr_st) {
+                const MVMStorageSpec *ss = attr_st->REPR->get_storage_spec(tc, attr_st);
+                if (ss->inlineable && ss->boxed_primitive == MVM_STORAGE_SPEC_BP_INT &&
+                        ss->bits / 8 == sizeof(AO_t))
+                    return (AO_t *)((char *)data + repr_data->attribute_offsets[slot]);
+            }
+            MVM_exception_throw_adhoc(tc,
+                "Can only do an atomic integer operation on an atomicint attribute");
+        }
+        else if (kind == MVM_reg_obj) {
+            return (AO_t *)((char *)data + repr_data->attribute_offsets[slot]);
+        }
+        else {
+            MVM_exception_throw_adhoc(tc,
+                "Can only perform atomic operations on object or atomicint attributes");
         }
-        MVM_exception_throw_adhoc(tc,
-            "Can only do an atomic integer operation on an atomicint attribute");
     }
     else {
         no_such_attribute(tc, "get atomic reference to", class_handle, name, st);
@@ -1235,6 +1245,9 @@ static void at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *d
     del = get_obj_at_offset(data, repr_data->attribute_offsets[repr_data->pos_del_slot]);
     REPR(del)->pos_funcs.at_pos(tc, STABLE(del), del, OBJECT_BODY(del), index, value, kind);
 }
+void MVM_P6opaque_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value, MVMuint16 kind) {
+    return at_pos(tc, st, root, data, index, value, kind);
+}
 
 static void bind_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister value, MVMuint16 kind) {
     MVMP6opaqueREPRData *repr_data = (MVMP6opaqueREPRData *)st->REPR_data;
@@ -1296,6 +1309,16 @@ static void shift(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *da
     REPR(del)->pos_funcs.shift(tc, STABLE(del), del, OBJECT_BODY(del), value, kind);
 }
 
+static void oslice(MVMThreadContext *tc, MVMSTable *st, MVMObject *src, void *data, MVMObject *dest, MVMint64 start, MVMint64 end) {
+    MVMP6opaqueREPRData *repr_data = (MVMP6opaqueREPRData *)st->REPR_data;
+    MVMObject *del;
+    if (repr_data->pos_del_slot == -1)
+        die_no_pos_del(tc, st);
+    data = MVM_p6opaque_real_data(tc, data);
+    del = get_obj_at_offset(data, repr_data->attribute_offsets[repr_data->pos_del_slot]);
+    REPR(del)->pos_funcs.slice(tc, STABLE(del), del, OBJECT_BODY(del), dest, start, end);
+}
+
 static void osplice(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *target_array, MVMint64 offset, MVMuint64 elems) {
     MVMP6opaqueREPRData *repr_data = (MVMP6opaqueREPRData *)st->REPR_data;
     MVMObject *del;
@@ -1626,6 +1649,7 @@ static const MVMREPROps P6opaque_this_repr = {
         pop,
         unshift,
         shift,
+        oslice,
         osplice,
         MVM_REPR_DEFAULT_AT_POS_MULTIDIM,
         MVM_REPR_DEFAULT_BIND_POS_MULTIDIM,
@@ -1682,6 +1706,11 @@ static void dump_p6opaque(MVMThreadContext *tc, MVMObject *obj, int nested) {
         MVMint16 const num_attributes = repr_data->num_attributes;
         MVMint16 cur_attribute = 0;
         MVMP6opaqueNameMap * const name_to_index_mapping = repr_data->name_to_index_mapping;
+        if (!IS_CONCRETE(obj)) {
+            fprintf(stderr, "(%s", MVM_6model_get_debug_name(tc, obj));
+            fprintf(stderr, nested ? ")" : ")\n");
+            return;
+        }
         fprintf(stderr, "%s.new(", MVM_6model_get_debug_name(tc, obj));
         if (name_to_index_mapping != NULL) {
             MVMint16 i;
@@ -1710,10 +1739,22 @@ static void dump_p6opaque(MVMThreadContext *tc, MVMObject *obj, int nested) {
                             }
                         }
                         else {
-                            MVMString * const s = attr_st->REPR->box_funcs.get_str(tc, attr_st, obj, (char *)data + offset);
-                            char * const str = MVM_string_utf8_encode_C_string(tc, s);
-                            fprintf(stderr, "='%s'", str);
-                            MVM_free(str);
+                            if (attr_st->REPR->ID == MVM_REPR_ID_P6str) {
+                                char * const str = MVM_string_utf8_encode_C_string(tc, (MVMString *)get_obj_at_offset(data, offset));
+                                fprintf(stderr, "='%s'", str);
+                                MVM_free(str);
+                            }
+                            else if (attr_st->REPR->ID == MVM_REPR_ID_P6int) {
+                                MVMint64 val = attr_st->REPR->box_funcs.get_int(tc, attr_st, obj, (char *)data + offset);
+                                fprintf(stderr, "=%ld", val);
+                            }
+                            else {
+                                fprintf(stderr, "[%d]=%s", repr_data->attribute_offsets[slot], MVM_6model_get_stable_debug_name(tc, attr_st));
+                            }
+                            /*MVMString * const s = attr_st->REPR->box_funcs.get_str(tc, attr_st, obj, (char *)data + offset);*/
+                            /*char * const str = MVM_string_utf8_encode_C_string(tc, s);*/
+                            /*fprintf(stderr, "='%s'", str);*/
+                            /*MVM_free(str);*/
                         }
                     }
                     if (cur_attribute++ < num_attributes - 1)
