@@ -1,12 +1,12 @@
 #include "moar.h"
 
 /* This representation's function pointer table. */
-static const MVMREPROps this_repr;
+static const MVMREPROps SCRef_this_repr;
 
 /* Creates a new type object of this representation, and associates it with
  * the given HOW. */
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
-    MVMSTable *st  = MVM_gc_allocate_stable(tc, &this_repr, HOW);
+    MVMSTable *st  = MVM_gc_allocate_stable(tc, &SCRef_this_repr, HOW);
 
     MVMROOT(tc, st, {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
@@ -79,6 +79,8 @@ static void gc_mark(MVMThreadContext *tc, MVMSTable *st, void *data, MVMGCWorkli
         MVM_gc_worklist_add(tc, worklist, &(sc->sr->root.string_comp_unit));
         MVM_gc_worklist_add(tc, worklist, &(sc->sr->codes_list));
         MVM_gc_worklist_add(tc, worklist, &(sc->sr->current_object));
+        for (i = 0; i < sc->sr->root.num_contexts; i++)
+            MVM_gc_worklist_add(tc, worklist, &(sc->sr->contexts[i]));
     }
 }
 
@@ -90,10 +92,10 @@ static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
         return;
 
     /* Remove from weakref lookup hash (which doesn't count as a root). */
-    uv_mutex_lock(&tc->instance->mutex_sc_weakhash);
+    uv_mutex_lock(&tc->instance->mutex_sc_registry);
     HASH_DELETE(hash_handle, tc->instance->sc_weakhash, sc->body);
     tc->instance->all_scs[sc->body->sc_idx] = NULL;
-    uv_mutex_unlock(&tc->instance->mutex_sc_weakhash);
+    uv_mutex_unlock(&tc->instance->mutex_sc_registry);
 
     /* Free manually managed object and STable root list memory. */
     MVM_free(sc->body->root_objects);
@@ -197,10 +199,10 @@ static void describe_refs(MVMThreadContext *tc, MVMHeapSnapshotState *ss, MVMSTa
 
 /* Initializes the representation. */
 const MVMREPROps * MVMSCRef_initialize(MVMThreadContext *tc) {
-    return &this_repr;
+    return &SCRef_this_repr;
 }
 
-static const MVMREPROps this_repr = {
+static const MVMREPROps SCRef_this_repr = {
     type_object_for,
     MVM_gc_allocate_object,
     initialize,
