@@ -1,4 +1,5 @@
 #include "moar.h"
+#include "limits.h"
 
 /* This representation's function pointer table. */
 static const MVMREPROps VMArray_this_repr;
@@ -103,7 +104,7 @@ static void gc_mark_repr_data(MVMThreadContext *tc, MVMSTable *st, MVMGCWorklist
     MVM_gc_worklist_add(tc, worklist, &repr_data->elem_type);
 }
 
-/* Marks the representation data in an STable.*/
+/* Frees the representation data in an STable.*/
 static void gc_free_repr_data(MVMThreadContext *tc, MVMSTable *st) {
     MVM_free(st->REPR_data);
 }
@@ -237,9 +238,12 @@ static void at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *d
                 value->i64 = (MVMint64)body->slots.u8[body->start + index];
             break;
         default:
-            MVM_exception_throw_adhoc(tc, "MVMArray: Unhandled slot type");
+            MVM_exception_throw_adhoc(tc, "MVMArray: Unhandled slot type, got '%s'", MVM_reg_get_debug_name(tc, repr_data->slot_type));
     }
 }
+void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value, MVMuint16 kind) {
+    return at_pos(tc, st, root, data, index, value, kind);
+}
 
 static MVMuint64 zero_slots(MVMThreadContext *tc, MVMArrayBody *body,
         MVMuint64 elems, MVMuint64 ssize, MVMuint8 slot_type) {
@@ -304,28 +308,29 @@ static void set_size_internal(MVMThreadContext *tc, MVMArrayBody *body, MVMuint6
     MVMuint64   ssize = body->ssize;
     void       *slots = body->slots.any;
 
-    if (n < 0)
-        MVM_exception_throw_adhoc(tc,
-            "MVMArray: Can't resize to negative elements");
-
     if (n == elems)
         return;
 
-    /* if there aren't enough slots at the end, shift off empty slots
-     * from the beginning first */
     if (start > 0 && n + start > ssize) {
+        /* if there aren't enough slots at the end, shift off empty slots
+         * from the beginning first */
         if (elems > 0)
             memmove(slots,
                 (char *)slots + start * repr_data->elem_size,
                 elems * repr_data->elem_size);
         body->start = 0;
         /* fill out any unused slots with NULL pointers or zero values */
-        elems = zero_slots(tc, body, elems, ssize, repr_data->slot_type);
+        zero_slots(tc, body, elems, start+elems, repr_data->slot_type);
+        elems = ssize; /* we'll use this as a point to clear from later */
+    }
+    else if (n < elems) {
+        /* we're downsizing; clear off extra slots */
+        zero_slots(tc, body, n+start, start+elems, repr_data->slot_type);
     }
 
-    body->elems = n;
     if (n <= ssize) {
         /* we already have n slots available, we can just return */
+        body->elems = n;
         return;
     }
 
@@ -341,10 +346,20 @@ static void set_size_internal(MVMThreadContext *tc, MVMArrayBody *body, MVMuint6
     else {
         ssize = (n + 0x1000) & ~0xfffUL;
     }
-    if (ssize > (1UL << (8 * sizeof(size_t) - repr_data->elem_size)))
-        MVM_exception_throw_adhoc(tc,
-            "Unable to allocate an array of %"PRIu64" elements",
-            ssize);
+    {
+        /* Our budget is 2^(
+         *     <number of bits in an array index>
+         *     - <number of bits to address individual bytes in an array element>
+         * ) */
+        size_t const elem_addr_size = repr_data->elem_size == 8 ? 4 :
+                                      repr_data->elem_size == 4 ? 3 :
+                                      repr_data->elem_size == 2 ? 2 :
+                                                                  1;
+        if (ssize > (1ULL << (CHAR_BIT * sizeof(size_t) - elem_addr_size)))
+            MVM_exception_throw_adhoc(tc,
+                "Unable to allocate an array of %"PRIu64" elements",
+                ssize);
+    }
 
     /* now allocate the new slot buffer */
     slots = (slots)
@@ -356,6 +371,8 @@ static void set_size_internal(MVMThreadContext *tc, MVMArrayBody *body, MVMuint6
     zero_slots(tc, body, elems, ssize, repr_data->slot_type);
 
     body->ssize = ssize;
+    /* set elems last so no thread tries to access slots before they are available */
+    body->elems = n;
 }
 
 static void bind_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister value, MVMuint16 kind) {
@@ -778,11 +795,95 @@ static void shift(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *da
     exit_single_user(tc, body);
 }
 
+static void copy_elements(MVMThreadContext *tc, MVMObject *src, MVMObject *dest, MVMint64 s_offset, MVMint64 d_offset, MVMint64 elems) {
+    MVMArrayBody     *s_body      = (MVMArrayBody *)OBJECT_BODY(src);
+    MVMArrayBody     *d_body      = (MVMArrayBody *)OBJECT_BODY(dest);
+    MVMArrayREPRData *s_repr_data = REPR(src)->ID == MVM_REPR_ID_VMArray
+                                    ? (MVMArrayREPRData *)STABLE(src)->REPR_data  : NULL;
+    MVMArrayREPRData *d_repr_data = REPR(src)->ID == MVM_REPR_ID_VMArray
+                                    ? (MVMArrayREPRData *)STABLE(dest)->REPR_data : NULL;
+
+    if (elems > 0) {
+        MVMint64  i;
+        MVMuint16 kind;
+        MVMuint8 d_needs_barrier = dest->header.flags & MVM_CF_SECOND_GEN;
+        if (s_repr_data && d_repr_data
+                && s_repr_data->slot_type == d_repr_data->slot_type
+                && s_repr_data->elem_size == d_repr_data->elem_size
+                && (d_repr_data->slot_type != MVM_ARRAY_OBJ || !d_needs_barrier)
+                && d_repr_data->slot_type  != MVM_ARRAY_STR) {
+            /* Optimized for copying from a VMArray with same slot type */
+            MVMint64 s_start = s_body->start;
+            MVMint64 d_start = d_body->start;
+            memcpy( d_body->slots.u8 + (d_start + d_offset) * d_repr_data->elem_size,
+                    s_body->slots.u8  + (s_start + s_offset) * s_repr_data->elem_size,
+                    d_repr_data->elem_size * elems
+            );
+        }
+        else {
+            switch (s_repr_data->slot_type) {
+                case MVM_ARRAY_OBJ:
+                    kind = MVM_reg_obj;
+                    break;
+                case MVM_ARRAY_STR:
+                    kind = MVM_reg_str;
+                    break;
+                case MVM_ARRAY_I64:
+                case MVM_ARRAY_I32:
+                case MVM_ARRAY_I16:
+                case MVM_ARRAY_I8:
+                    kind = MVM_reg_int64;
+                    break;
+                case MVM_ARRAY_N64:
+                case MVM_ARRAY_N32:
+                    kind = MVM_reg_num64;
+                    break;
+                case MVM_ARRAY_U64:
+                case MVM_ARRAY_U32:
+                case MVM_ARRAY_U16:
+                case MVM_ARRAY_U8:
+                    kind = MVM_reg_int64;
+                    break;
+                default:
+                    abort(); /* never reached, silence compiler warnings */
+            }
+            for (i = 0; i < elems; i++) {
+                MVMRegister to_copy;
+                REPR(src)->pos_funcs.at_pos(tc, STABLE(src), src, s_body, s_offset + i, &to_copy, kind);
+                bind_pos(tc, STABLE(dest), dest, d_body, d_offset + i, to_copy, kind);
+            }
+        }
+    }
+}
+
+static void aslice(MVMThreadContext *tc, MVMSTable *st, MVMObject *src, void *data, MVMObject *dest, MVMint64 start, MVMint64 end) {
+    MVMArrayBody     *s_body      = (MVMArrayBody *)data;
+    MVMArrayBody     *d_body      = (MVMArrayBody *)OBJECT_BODY(dest);
+    MVMArrayREPRData *d_repr_data = REPR(dest)->ID == MVM_REPR_ID_VMArray
+                                      ? STABLE(dest)->REPR_data : NULL;
+
+    MVMint64 total_elems = REPR(src)->elems(tc, st, src, s_body);
+    MVMint64 elems;
+
+    start = start < 0 ? total_elems + start : start;
+    end   = end   < 0 ? total_elems + end   : end;
+    if ( end < start || start < 0 || end < 0 || total_elems <= start || total_elems <= end ) {
+        MVM_exception_throw_adhoc(tc, "MVMArray: Slice index out of bounds");
+    }
+
+    elems = end - start + 1;
+    if (d_repr_data) {
+        set_size_internal(tc, d_body, elems, d_repr_data);
+    }
+
+    copy_elements(tc, src, dest, start, 0, elems);
+}
+
 /* This whole splice optimization can be optimized for the case we have two
  * MVMArray representation objects. */
 static void asplice(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMObject *from, MVMint64 offset, MVMuint64 count) {
-    MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
-    MVMArrayBody     *body      = (MVMArrayBody *)data;
+    MVMArrayREPRData *repr_data   = (MVMArrayREPRData *)st->REPR_data;
+    MVMArrayBody     *body        = (MVMArrayBody *)data;
 
     MVMint64 elems0 = body->elems;
     MVMint64 elems1 = REPR(from)->elems(tc, STABLE(from), from, OBJECT_BODY(from));
@@ -858,43 +959,9 @@ static void asplice(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *
     }
     exit_single_user(tc, body);
 
+
     /* now copy C<from>'s elements into SELF */
-    if (elems1 > 0) {
-        MVMint64  i;
-        MVMuint16 kind;
-        switch (repr_data->slot_type) {
-            case MVM_ARRAY_OBJ:
-                kind = MVM_reg_obj;
-                break;
-            case MVM_ARRAY_STR:
-                kind = MVM_reg_str;
-                break;
-            case MVM_ARRAY_I64:
-            case MVM_ARRAY_I32:
-            case MVM_ARRAY_I16:
-            case MVM_ARRAY_I8:
-                kind = MVM_reg_int64;
-                break;
-            case MVM_ARRAY_N64:
-            case MVM_ARRAY_N32:
-                kind = MVM_reg_num64;
-                break;
-            case MVM_ARRAY_U64:
-            case MVM_ARRAY_U32:
-            case MVM_ARRAY_U16:
-            case MVM_ARRAY_U8:
-                kind = MVM_reg_int64;
-                break;
-            default:
-                abort(); /* never reached, silence compiler warnings */
-        }
-        for (i = 0; i < elems1; i++) {
-            MVMRegister to_copy;
-            REPR(from)->pos_funcs.at_pos(tc, STABLE(from), from,
-                OBJECT_BODY(from), i, &to_copy, kind);
-            bind_pos(tc, st, root, data, offset + i, to_copy, kind);
-        }
-    }
+    copy_elements(tc, from, root, 0, offset, elems1);
 }
 
 static void at_pos_multidim(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 num_indices, MVMint64 *indices, MVMRegister *result, MVMuint16 kind) {
@@ -968,6 +1035,35 @@ static MVMStorageSpec get_elem_storage_spec(MVMThreadContext *tc, MVMSTable *st)
     return spec;
 }
 
+static AO_t * pos_as_atomic(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
+                            void *data, MVMint64 index) {
+    MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes and require in bounds. */
+    if (index < 0)
+        index += body->elems;
+    if (index < 0 || index >= body->elems)
+        MVM_exception_throw_adhoc(tc, "Index out of bounds in atomic operation on array");
+
+    if (sizeof(AO_t) == 8 && (repr_data->slot_type == MVM_ARRAY_I64 ||
+            repr_data->slot_type == MVM_ARRAY_U64))
+        return (AO_t *)&(body->slots.i64[body->start + index]);
+    if (sizeof(AO_t) == 4 && (repr_data->slot_type == MVM_ARRAY_I32 ||
+            repr_data->slot_type == MVM_ARRAY_U32))
+        return (AO_t *)&(body->slots.i32[body->start + index]);
+    MVM_exception_throw_adhoc(tc,
+        "Can only do integer atomic operation on native integer array element of atomic size");
+}
+
+static AO_t * pos_as_atomic_multidim(MVMThreadContext *tc, MVMSTable *st, MVMObject *root,
+                                     void *data, MVMint64 num_indices, MVMint64 *indices) {
+    if (num_indices != 1)
+        MVM_exception_throw_adhoc(tc,
+            "A dynamic array can only be indexed with a single dimension");
+    return pos_as_atomic(tc, st, root, data, indices[0]);
+}
+
 /* Compose the representation. */
 static void spec_to_repr_data(MVMThreadContext *tc, MVMArrayREPRData *repr_data, const MVMStorageSpec *spec) {
     switch (spec->boxed_primitive) {
@@ -1289,12 +1385,15 @@ static const MVMREPROps VMArray_this_repr = {
         pop,
         unshift,
         shift,
+        aslice,
         asplice,
         at_pos_multidim,
         bind_pos_multidim,
         dimensions,
         set_dimensions,
-        get_elem_storage_spec
+        get_elem_storage_spec,
+        pos_as_atomic,
+        pos_as_atomic_multidim
     },    /* pos_funcs */
     MVM_REPR_DEFAULT_ASS_FUNCS,
     elems,
