@@ -12,14 +12,14 @@ MVMObject * MVM_sc_create(MVMThreadContext *tc, MVMString *handle) {
         sc = (MVMSerializationContext *)REPR(tc->instance->SCRef)->allocate(tc, STABLE(tc->instance->SCRef));
         MVMROOT(tc, sc, {
             /* Add to weak lookup hash. */
-            uv_mutex_lock(&tc->instance->mutex_sc_weakhash);
+            uv_mutex_lock(&tc->instance->mutex_sc_registry);
             MVM_HASH_GET(tc, tc->instance->sc_weakhash, handle, scb);
             if (!scb) {
                 sc->body = scb = MVM_calloc(1, sizeof(MVMSerializationContextBody));
                 MVM_ASSIGN_REF(tc, &(sc->common.header), scb->handle, handle);
                 MVM_HASH_BIND(tc, tc->instance->sc_weakhash, handle, scb);
                 /* Calling repr_init will allocate, BUT if it does so, and we
-                 * get unlucky, the GC will try to acquire mutex_sc_weakhash.
+                 * get unlucky, the GC will try to acquire mutex_sc_registry.
                  * This deadlocks. Thus, we force allocation in gen2, which
                  * can never trigger GC. Note that releasing the mutex early
                  * is not a good way to fix this, as it leaves a race to
@@ -43,7 +43,7 @@ MVMObject * MVM_sc_create(MVMThreadContext *tc, MVMString *handle) {
                 MVM_repr_init(tc, (MVMObject *)sc);
                 MVM_gc_allocate_gen2_default_clear(tc);
             }
-            uv_mutex_unlock(&tc->instance->mutex_sc_weakhash);
+            uv_mutex_unlock(&tc->instance->mutex_sc_registry);
         });
     });
 
@@ -51,19 +51,26 @@ MVMObject * MVM_sc_create(MVMThreadContext *tc, MVMString *handle) {
 }
 
 /* Makes an entry in all SCs list, the index of which is used to refer to
- * SCs in object headers. */
+ * SCs in object headers. This must only be called while holding the SC
+ * registry mutex. However, the all SCs list is read without the lock.
+ * Thus we allocate memory using the FSA and free it at a safepoint. */
 void MVM_sc_add_all_scs_entry(MVMThreadContext *tc, MVMSerializationContextBody *scb) {
     if (tc->instance->all_scs_next_idx == tc->instance->all_scs_alloc) {
-        tc->instance->all_scs_alloc += 32;
         if (tc->instance->all_scs_next_idx == 0) {
             /* First time; allocate, and NULL first slot as it is
              * the "no SC" sentinel value. */
-            tc->instance->all_scs    = MVM_malloc(tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
+            tc->instance->all_scs_alloc = 32;
+            tc->instance->all_scs    = MVM_fixed_size_alloc(tc, tc->instance->fsa,
+                tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
             tc->instance->all_scs[0] = NULL;
             tc->instance->all_scs_next_idx++;
         }
         else {
-            tc->instance->all_scs = MVM_realloc(tc->instance->all_scs,
+            MVMuint32 orig_alloc = tc->instance->all_scs_alloc;
+            tc->instance->all_scs_alloc += 32;
+            tc->instance->all_scs = MVM_fixed_size_realloc_at_safepoint(tc,
+                tc->instance->fsa, tc->instance->all_scs,
+                orig_alloc * sizeof(MVMSerializationContextBody *),
                 tc->instance->all_scs_alloc * sizeof(MVMSerializationContextBody *));
         }
     }
@@ -176,11 +183,22 @@ MVM_STATIC_INLINE MVMint64 sc_working(MVMSerializationContext *sc) {
     return sr && sr->working;
 }
 
+MVMuint8 MVM_sc_is_object_immediately_available(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx) {
+    MVMObject **roots = sc->body->root_objects;
+    MVMint64    count = sc->body->num_objects;
+    if (idx >= 0 && idx < count) {
+        if (roots[idx] && !sc_working(sc)) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
 /* Given an SC and an index, fetch the object stored there. */
 MVMObject * MVM_sc_get_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx) {
     MVMObject **roots = sc->body->root_objects;
     MVMint64    count = sc->body->num_objects;
-    if (idx >= 0 && idx < count)
+    if (MVM_LIKELY(idx >= 0 && idx < count))
         return roots[idx] && !sc_working(sc)
             ? roots[idx]
             : MVM_serialization_demand_object(tc, sc, idx);
@@ -193,14 +211,6 @@ MVMObject * MVM_sc_get_object(MVMThreadContext *tc, MVMSerializationContext *sc,
     }
 }
 
-MVMObject * MVM_sc_get_sc_object(MVMThreadContext *tc, MVMCompUnit *cu,
-                                 MVMuint16 dep, MVMuint64 idx) {
-    MVMSerializationContext *sc = MVM_sc_get_sc(tc, cu, dep);
-    if (sc == NULL)
-        MVM_exception_throw_adhoc(tc, "SC not yet resolved; lookup failed");
-    return MVM_sc_get_object(tc, sc, idx);
-}
-
 /* Given an SC and an index, fetch the object stored there, or return NULL if
  * there is none. Does not cause lazy deserialization. */
 MVMObject * MVM_sc_try_get_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx) {
@@ -237,7 +247,7 @@ void MVM_sc_set_object(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint
 
 /* Given an SC and an index, fetch the STable stored there. */
 MVMSTable * MVM_sc_get_stable(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx) {
-    if (idx >= 0 && idx < sc->body->num_stables) {
+    if (MVM_LIKELY(idx >= 0 && idx < sc->body->num_stables)) {
         MVMSTable *got = sc->body->root_stables[idx];
         return got && !sc_working(sc) ? got : MVM_serialization_demand_stable(tc, sc, idx);
     }
@@ -261,7 +271,7 @@ MVMSTable * MVM_sc_try_get_stable(MVMThreadContext *tc, MVMSerializationContext
 
 /* Given an SC, an index, and an STable, store the STable at the index. */
 void MVM_sc_set_stable(MVMThreadContext *tc, MVMSerializationContext *sc, MVMint64 idx, MVMSTable *st) {
-    if (idx < 0)
+    if (MVM_UNLIKELY(idx < 0))
         MVM_exception_throw_adhoc(tc,
             "Invalid (negative) STable index %"PRId64, idx);
     if (idx < sc->body->num_stables) {
@@ -319,9 +329,9 @@ MVMObject * MVM_sc_get_code(MVMThreadContext *tc, MVMSerializationContext *sc, M
 /* Resolves an SC handle using the SC weakhash. */
 MVMSerializationContext * MVM_sc_find_by_handle(MVMThreadContext *tc, MVMString *handle) {
     MVMSerializationContextBody *scb;
-    uv_mutex_lock(&tc->instance->mutex_sc_weakhash);
+    uv_mutex_lock(&tc->instance->mutex_sc_registry);
     MVM_HASH_GET(tc, tc->instance->sc_weakhash, handle, scb);
-    uv_mutex_unlock(&tc->instance->mutex_sc_weakhash);
+    uv_mutex_unlock(&tc->instance->mutex_sc_registry);
     return scb && scb->sc ? scb->sc : NULL;
 }
 
