@@ -1,4 +1,5 @@
 #include "moar.h"
+#include "math/grisu.h"
 
 #if defined(_MSC_VER)
 #define strtoll _strtoi64
@@ -170,6 +171,30 @@ MVMString * MVM_coerce_i_s(MVMThreadContext *tc, MVMint64 i) {
     }
 }
 
+MVMString * MVM_coerce_u_s(MVMThreadContext *tc, MVMuint64 i) {
+    char buffer[64];
+    int len;
+    /* See if we can hit the cache. */
+    int cache = 0 <= i && i < MVM_INT_TO_STR_CACHE_SIZE;
+    if (cache) {
+        MVMString *cached = tc->instance->int_to_str_cache[i];
+        if (cached)
+            return cached;
+    }
+
+    /* Otherwise, need to do the work; cache it if in range. */
+    len = snprintf(buffer, 64, "%"PRIu64"", i);
+    if (0 <= len) {
+        MVMString *result = MVM_string_ascii_decode(tc, tc->instance->VMString, buffer, len);
+        if (cache)
+            tc->instance->int_to_str_cache[i] = result;
+        return result;
+    }
+    else {
+        MVM_exception_throw_adhoc(tc, "Could not stringify integer");
+    }
+}
+
 MVMString * MVM_coerce_n_s(MVMThreadContext *tc, MVMnum64 n) {
     if (n == MVM_num_posinf(tc)) {
         return MVM_string_ascii_decode_nt(tc, tc->instance->VMString, "Inf");
@@ -183,16 +208,8 @@ MVMString * MVM_coerce_n_s(MVMThreadContext *tc, MVMnum64 n) {
     else {
         char buf[64];
         int i;
-        if (snprintf(buf, 64, "%.15g", n) < 0)
+        if (dtoa_grisu3(n, buf, 64) < 0)
             MVM_exception_throw_adhoc(tc, "Could not stringify number");
-        if (strstr(buf, ".")) {
-            MVMint64 is_not_scientific = !strstr(buf, "e");
-            i = strlen(buf);
-            while (i > 1 && ((buf[--i] == '0'  && is_not_scientific) || buf[i] == ' '))
-                buf[i] = '\0';
-            if (buf[i] == '.')
-                buf[i] = '\0';
-        }
         return MVM_string_ascii_decode(tc, tc->instance->VMString, buf, strlen(buf));
     }
 }
@@ -431,13 +448,13 @@ void MVM_box_str(MVMThreadContext *tc, MVMString *value, MVMObject *type,
                  MVMRegister * dst) {
     MVMObject *box;
     MVMROOT(tc, value, {
-            box = REPR(type)->allocate(tc, STABLE(type));
-            if (REPR(box)->initialize)
-                REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
-            REPR(box)->box_funcs.set_str(tc, STABLE(box), box,
-                                         OBJECT_BODY(box), value);
-            dst->o = box;
-        });
+        box = REPR(type)->allocate(tc, STABLE(type));
+        if (REPR(box)->initialize)
+            REPR(box)->initialize(tc, STABLE(box), box, OBJECT_BODY(box));
+        REPR(box)->box_funcs.set_str(tc, STABLE(box), box,
+                                     OBJECT_BODY(box), value);
+        dst->o = box;
+    });
 }
 
 void MVM_box_uint(MVMThreadContext *tc, MVMuint64 value, MVMObject *type,
