@@ -51,6 +51,10 @@ void MVM_continuation_control(MVMThreadContext *tc, MVMint64 protect,
     MVMFrame           *root_frame  = NULL;
     MVMContinuationTag *tag_record  = NULL;
     MVMFrame            *jump_frame;
+
+
+    MVM_jit_code_trampoline(tc);
+
     MVMROOT2(tc, tag, code, {
         jump_frame = MVM_frame_force_to_heap(tc, tc->cur_frame);
     });
@@ -136,6 +140,10 @@ void MVM_continuation_control(MVMThreadContext *tc, MVMint64 protect,
 
 void MVM_continuation_invoke(MVMThreadContext *tc, MVMContinuation *cont,
                              MVMObject *code, MVMRegister *res_reg) {
+    /* First of all do a repr id check */
+    if (REPR(cont)->ID != MVM_REPR_ID_MVMContinuation)
+        MVM_exception_throw_adhoc(tc, "continuationinvoke expects an MVMContinuation");
+
     /* Ensure we are the only invoker of the continuation. */
     if (!MVM_trycas(&(cont->body.invoked), 0, 1))
         MVM_exception_throw_adhoc(tc, "This continuation has already been invoked");
@@ -151,6 +159,8 @@ void MVM_continuation_invoke(MVMThreadContext *tc, MVMContinuation *cont,
     tc->cur_frame->return_type = MVM_RETURN_OBJ;
     tc->cur_frame->return_address = *(tc->interp_cur_op);
 
+    MVM_jit_code_trampoline(tc);
+
     /* Switch to the target frame. */
     tc->cur_frame = cont->body.top;
     tc->current_frame_nr = cont->body.top->sequence_nr;
