@@ -6,14 +6,12 @@ int MVM_ext_load(MVMThreadContext *tc, MVMString *lib, MVMString *ext) {
     MVMDLLSym *sym;
     void (*init)(MVMThreadContext *);
 
-    MVMROOT(tc, lib, {
-    MVMROOT(tc, ext, {
+    MVMROOT2(tc, lib, ext, {
         colon = MVM_string_ascii_decode_nt(
             tc, tc->instance->VMString, ":");
         prefix = MVM_string_concatenate(tc, lib, colon);
         name = MVM_string_concatenate(tc, prefix, ext);
     });
-    });
 
     uv_mutex_lock(&tc->instance->mutex_ext_registry);
 
@@ -157,8 +155,10 @@ int MVM_ext_register_extop(MVMThreadContext *tc, const char *cname,
     entry->info.num_operands = num_operands;
     entry->info.pure         = flags & MVM_EXTOP_PURE;
     entry->info.deopt_point  = 0;
+    entry->info.logged       = 0;
     entry->info.no_inline    = flags & MVM_EXTOP_NOINLINE;
     entry->info.jittivity    = (flags & MVM_EXTOP_INVOKISH) ? MVM_JIT_INFO_INVOKISH : 0;
+    entry->info.uses_hll     = 0;
     memcpy(entry->info.operands, operands, num_operands);
     memset(entry->info.operands + num_operands, 0,
             MVM_MAX_OPERANDS - num_operands);
