@@ -352,8 +352,8 @@ struct MVMInstance {
     /* Cache the environment hash */
     MVMObject      *env_hash;
 
-    /* Cache the signal hash */
-    MVMObject      *sig_hash;
+    /* Cache the signal array */
+    MVMObject      *sig_arr;
 
     /* Flags indicating the signals available on the host system */
     MVMuint64       valid_sigs;
