@@ -265,6 +265,11 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
                 MVM_gc_root_temp_push(tc, (MVMCollectable **)&(args[i - 1].o));
                 num_roots++;
                 break;
+            case MVM_NATIVECALL_ARG_CPPSTRUCT:
+                args[i - 1].o = MVM_nativecall_make_cppstruct(tc, type, *(void **)cb_args[i - 1]);
+                MVM_gc_root_temp_push(tc, (MVMCollectable **)&(args[i - 1].o));
+                num_roots++;
+                break;
             case MVM_NATIVECALL_ARG_CPOINTER:
                 args[i - 1].o = MVM_nativecall_make_cpointer(tc, type, *(void **)cb_args[i - 1]);
                 MVM_gc_root_temp_push(tc, (MVMCollectable **)&(args[i - 1].o));
@@ -322,6 +327,9 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
         MVMCompUnit **backup_interp_cu          = tc->interp_cu;
         MVMFrame *backup_cur_frame              = MVM_frame_force_to_heap(tc, tc->cur_frame);
         MVMFrame *backup_thread_entry_frame     = tc->thread_entry_frame;
+        void **backup_jit_return_address        = tc->jit_return_address;
+        tc->jit_return_address                  = NULL;
+
         MVMROOT2(tc, backup_cur_frame, backup_thread_entry_frame, {
             MVMuint32 backup_mark                   = MVM_gc_root_temp_mark(tc);
             jmp_buf backup_interp_jump;
@@ -338,6 +346,7 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
             tc->cur_frame             = backup_cur_frame;
             tc->current_frame_nr      = backup_cur_frame->sequence_nr;
             tc->thread_entry_frame    = backup_thread_entry_frame;
+            tc->jit_return_address    = backup_jit_return_address;
             memcpy(tc->interp_jump, backup_interp_jump, sizeof(jmp_buf));
             MVM_gc_root_temp_mark_reset(tc, backup_mark);
         });
@@ -353,16 +362,16 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
         case MVM_NATIVECALL_ARG_VOID:
             break;
         case MVM_NATIVECALL_ARG_CHAR:
-            *(signed char *)cb_result = MVM_nativecall_unmarshal_char(tc, res.o);
+            *(ffi_sarg *)cb_result = MVM_nativecall_unmarshal_char(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_SHORT:
-            *(signed short *)cb_result = MVM_nativecall_unmarshal_short(tc, res.o);
+            *(ffi_sarg *)cb_result = MVM_nativecall_unmarshal_short(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_INT:
-            *(signed int *)cb_result = MVM_nativecall_unmarshal_int(tc, res.o);
+            *(ffi_sarg *)cb_result = MVM_nativecall_unmarshal_int(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_LONG:
-            *(signed long *)cb_result = MVM_nativecall_unmarshal_long(tc, res.o);
+            *(ffi_sarg *)cb_result = MVM_nativecall_unmarshal_long(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_LONGLONG:
             *(signed long long *)cb_result = MVM_nativecall_unmarshal_longlong(tc, res.o);
@@ -381,6 +390,9 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
         case MVM_NATIVECALL_ARG_CSTRUCT:
             *(void **)cb_result = MVM_nativecall_unmarshal_cstruct(tc, res.o);
             break;
+        case MVM_NATIVECALL_ARG_CPPSTRUCT:
+            *(void **)cb_result = MVM_nativecall_unmarshal_cppstruct(tc, res.o);
+            break;
         case MVM_NATIVECALL_ARG_CPOINTER:
             *(void **)cb_result = MVM_nativecall_unmarshal_cpointer(tc, res.o);
             break;
@@ -397,16 +409,16 @@ static void callback_handler(ffi_cif *cif, void *cb_result, void **cb_args, void
             *(void **)cb_result = unmarshal_callback(tc, res.o, data->types[0]);
             break;
         case MVM_NATIVECALL_ARG_UCHAR:
-            *(unsigned char *)cb_result = MVM_nativecall_unmarshal_uchar(tc, res.o);
+            *(ffi_arg *)cb_result = MVM_nativecall_unmarshal_uchar(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_USHORT:
-            *(unsigned short *)cb_result = MVM_nativecall_unmarshal_ushort(tc, res.o);
+            *(ffi_arg *)cb_result = MVM_nativecall_unmarshal_ushort(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_UINT:
-            *(unsigned int *)cb_result = MVM_nativecall_unmarshal_uint(tc, res.o);
+            *(ffi_arg *)cb_result = MVM_nativecall_unmarshal_uint(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_ULONG:
-            *(unsigned long *)cb_result = MVM_nativecall_unmarshal_ulong(tc, res.o);
+            *(ffi_arg *)cb_result = MVM_nativecall_unmarshal_ulong(tc, res.o);
             break;
         case MVM_NATIVECALL_ARG_ULONGLONG:
             *(unsigned long long *)cb_result = MVM_nativecall_unmarshal_ulonglong(tc, res.o);
