@@ -213,6 +213,10 @@ struct MVMThreadContext {
     MVMint32 current_frame_nr;
     MVMint32 next_frame_nr;
 
+    /* JIT return address pointer, so we can figure out the current position in
+     * the code */
+    void **jit_return_address;
+
     /* This thread's current spesh log to write in to, if there curently is
      * one. */
     MVMSpeshLog *spesh_log;
@@ -246,6 +250,12 @@ struct MVMThreadContext {
     MVMint32 osr_hunt_frame_nr;
     MVMint32 osr_hunt_num_spesh_candidates;
 
+    /* If we are currently in a spesh plugin, the current set of guards we
+     * have recorded. */
+    MVMSpeshPluginGuard *plugin_guards;
+    MVMObject *plugin_guard_args;
+    MVMuint32 num_plugin_guards;
+
     /************************************************************************
      * Per-thread state held by assorted VM subsystems
      ************************************************************************/
