@@ -120,6 +120,9 @@ void MVM_gc_root_add_instance_roots_to_worklist(MVMThreadContext *tc, MVMGCWorkl
     add_collectable(tc, worklist, snapshot, tc->instance->env_hash,
         "Cached environment variable hash");
 
+    add_collectable(tc, worklist, snapshot, tc->instance->sig_arr,
+        "Cached signal mapping array");
+
     MVM_debugserver_mark_handles(tc, worklist, snapshot);
 }
 
@@ -184,13 +187,16 @@ void MVM_gc_root_add_tc_roots_to_worklist(MVMThreadContext *tc, MVMGCWorklist *w
     add_collectable(tc, worklist, snapshot, tc->serialized_string_heap,
         "Serialized string heap");
 
-    /* Specialization log and stack simulation. */
+    /* Specialization log, stack simulation, and plugin state. */
     add_collectable(tc, worklist, snapshot, tc->spesh_log, "Specialization log");
     if (worklist)
         MVM_spesh_sim_stack_gc_mark(tc, tc->spesh_sim_stack, worklist);
     else {
         MVM_spesh_sim_stack_gc_describe(tc, snapshot, tc->spesh_sim_stack);
     }
+    MVM_spesh_plugin_guard_list_mark(tc, tc->plugin_guards, tc->num_plugin_guards, worklist);
+    add_collectable(tc, worklist, snapshot, tc->plugin_guard_args,
+        "Plugin guard args");
 
     if (tc->step_mode_frame)
         add_collectable(tc, worklist, snapshot, tc->step_mode_frame,
@@ -269,7 +275,7 @@ void MVM_gc_root_gen2_add(MVMThreadContext *tc, MVMCollectable *c) {
     if (tc->num_gen2roots == tc->alloc_gen2roots) {
         tc->alloc_gen2roots *= 2;
         tc->gen2roots = MVM_realloc(tc->gen2roots,
-            sizeof(MVMCollectable **) * tc->alloc_gen2roots);
+            sizeof(MVMCollectable *) * tc->alloc_gen2roots);
     }
 
     /* Add this one to the list. */
