@@ -13,12 +13,6 @@
 /* platform-specific setjmp override */
 #include <platform/setjmp.h>
 
-/* stuff for uthash */
-#define uthash_fatal(msg) MVM_exception_throw_adhoc(tc, "internal hash error: " msg)
-typedef uint32_t MVMhashv;
-
-#include "strings/uthash.h"
-
 /* libuv
  * must precede atomic_ops.h so we get the ordering of Winapi headers right
  */
@@ -88,6 +82,12 @@ typedef double   MVMnum64;
 #  define MVM_USED_BY_JIT
 #endif
 
+/* stuff for uthash */
+#define uthash_fatal(msg) MVM_exception_throw_adhoc(tc, "internal hash error: " msg)
+typedef uint32_t MVMhashv;
+
+#include "strings/uthash.h"
+
 MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 
 /* Headers for various other data structures and APIs. */
@@ -162,6 +162,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "spesh/stats.h"
 #include "spesh/plan.h"
 #include "spesh/arg_guard.h"
+#include "spesh/plugin.h"
 #include "strings/nfg.h"
 #include "strings/normalize.h"
 #include "strings/decode_stream.h"
@@ -200,6 +201,7 @@ MVM_PUBLIC const MVMint32 MVM_jit_support(void);
 #include "jit/tile.h"
 #include "jit/compile.h"
 #include "jit/log.h"
+#include "jit/interface.h"
 #include "profiler/instrument.h"
 #include "profiler/log.h"
 #include "profiler/profile.h"
@@ -214,7 +216,7 @@ MVMObject *MVM_backend_config(MVMThreadContext *tc);
 MVM_PUBLIC MVMInstance * MVM_vm_create_instance(void);
 MVM_PUBLIC void MVM_vm_run_file(MVMInstance *instance, const char *filename);
 MVM_PUBLIC void MVM_vm_dump_file(MVMInstance *instance, const char *filename);
-MVM_PUBLIC MVM_NO_RETURN void MVM_vm_exit(MVMInstance *instance) MVM_NO_RETURN_GCC;
+MVM_PUBLIC MVM_NO_RETURN void MVM_vm_exit(MVMInstance *instance) MVM_NO_RETURN_ATTRIBUTE;
 MVM_PUBLIC void MVM_vm_destroy_instance(MVMInstance *instance);
 MVM_PUBLIC void MVM_vm_set_clargs(MVMInstance *instance, int argc, char **argv);
 MVM_PUBLIC void MVM_vm_set_exec_name(MVMInstance *instance, const char *exec_name);
