@@ -33,6 +33,19 @@ static const MVMOpInfo * get_op_info(MVMThreadContext *tc, MVMCompUnit *cu, MVMu
     }
 }
 
+/* Grows the spesh graph's deopt table if it is already full, so that we have
+ * space for 1 more entry. */
+void MVM_spesh_graph_grow_deopt_table(MVMThreadContext *tc, MVMSpeshGraph *g) {
+    if (g->num_deopt_addrs == g->alloc_deopt_addrs) {
+        g->alloc_deopt_addrs += 8;
+        if (g->deopt_addrs)
+            g->deopt_addrs = MVM_realloc(g->deopt_addrs,
+                g->alloc_deopt_addrs * sizeof(MVMint32) * 2);
+        else
+            g->deopt_addrs = MVM_malloc(g->alloc_deopt_addrs * sizeof(MVMint32) * 2);
+    }
+}
+
 /* Records a de-optimization annotation and mapping pair. */
 void MVM_spesh_graph_add_deopt_annotation(MVMThreadContext *tc, MVMSpeshGraph *g,
                                           MVMSpeshIns *ins_node, MVMuint32 deopt_target,
@@ -45,14 +58,7 @@ void MVM_spesh_graph_add_deopt_annotation(MVMThreadContext *tc, MVMSpeshGraph *g
     ins_node->annotations = ann;
 
     /* Record PC in the deopt entries table. */
-    if (g->num_deopt_addrs == g->alloc_deopt_addrs) {
-        g->alloc_deopt_addrs += 4;
-        if (g->deopt_addrs)
-            g->deopt_addrs = MVM_realloc(g->deopt_addrs,
-                g->alloc_deopt_addrs * sizeof(MVMint32) * 2);
-        else
-            g->deopt_addrs = MVM_malloc(g->alloc_deopt_addrs * sizeof(MVMint32) * 2);
-    }
+    MVM_spesh_graph_grow_deopt_table(tc, g);
     g->deopt_addrs[2 * g->num_deopt_addrs] = deopt_target;
     g->num_deopt_addrs++;
 }
@@ -690,13 +696,17 @@ static void insert_object_null_instructions(MVMThreadContext *tc, MVMSpeshGraph
     MVMuint16 *local_types = g->sf->body.local_types;
     MVMuint16  num_locals = g->sf->body.num_locals;
     MVMuint16 i;
+    MVMSpeshIns *insert_after = NULL;
+    if (insert_bb->first_ins && insert_bb->first_ins->info->opcode == MVM_OP_prof_enter) {
+        insert_after = insert_bb->first_ins;
+    }
     for (i = 0; i < num_locals; i++) {
         if (local_types[i] == MVM_reg_obj && !is_handler_reg(tc, g, i)) {
             MVMSpeshIns *null_ins = MVM_spesh_alloc(tc, g, sizeof(MVMSpeshIns));
             null_ins->info = MVM_op_get_op(MVM_OP_null);
             null_ins->operands = MVM_spesh_alloc(tc, g, sizeof(MVMSpeshOperand));
             null_ins->operands[0].reg.orig = i;
-            MVM_spesh_manipulate_insert_ins(tc, insert_bb, NULL, null_ins);
+            MVM_spesh_manipulate_insert_ins(tc, insert_bb, insert_after, null_ins);
         }
     }
 }
@@ -1251,11 +1261,15 @@ MVMSpeshGraph * MVM_spesh_graph_create_from_cand(MVMThreadContext *tc, MVMStatic
     g->deopt_named_used_bit_field = cand->deopt_named_used_bit_field;
     g->local_types       = cand->local_types;
     g->lexical_types     = cand->lexical_types;
-    g->spesh_slots       = cand->spesh_slots;
     g->num_spesh_slots   = cand->num_spesh_slots;
+    g->alloc_spesh_slots = cand->num_spesh_slots;
     g->phi_infos         = MVM_spesh_alloc(tc, g, MVMPhiNodeCacheSize * sizeof(MVMOpInfo));
     g->cand              = cand;
 
+    g->spesh_slots       = MVM_malloc(g->alloc_spesh_slots * sizeof(MVMCollectable *));
+
+    memcpy(g->spesh_slots, cand->spesh_slots, sizeof(MVMCollectable *) * g->num_spesh_slots);
+
     /* Ensure the frame is validated, since we'll rely on this. */
     if (sf->body.instrumentation_level == 0) {
         MVM_spesh_graph_destroy(tc, g);
