@@ -49,9 +49,9 @@ static const MVMuint8 utf8d[] = {
   12,36,12,12,12,12,12,12,12,12,12,12,
 };
 
-static MVMint32
+MVM_STATIC_INLINE MVMint32
 decode_utf8_byte(MVMint32 *state, MVMGrapheme32 *codep, MVMuint8 byte) {
-  MVMint32 type = utf8d[byte];
+  const MVMint32 type = utf8d[byte];
 
   *codep = (*state != UTF8_ACCEPT) ?
     (byte & 0x3fu) | (*codep << 6) :
@@ -177,8 +177,6 @@ MVMString * MVM_string_utf8_decode(MVMThreadContext *tc, const MVMObject *result
     MVMint32 line_ending = 0;
     MVMint32 state = 0;
     MVMint32 bufsize = bytes;
-    MVMGrapheme32 lowest_graph  =  0x7fffffff;
-    MVMGrapheme32 highest_graph = -0x7fffffff;
     MVMGrapheme32 *buffer = MVM_malloc(sizeof(MVMGrapheme32) * bufsize);
     size_t orig_bytes;
     const char *orig_utf8;
@@ -194,7 +192,7 @@ MVMString * MVM_string_utf8_decode(MVMThreadContext *tc, const MVMObject *result
     orig_utf8 = utf8;
 
     for (; bytes; ++utf8, --bytes) {
-        switch(decode_utf8_byte(&state, &codepoint, (MVMuint8)*utf8)) {
+        switch(MVM_EXPECT(decode_utf8_byte(&state, &codepoint, (MVMuint8)*utf8), UTF8_ACCEPT)) {
         case UTF8_ACCEPT: { /* got a codepoint */
             MVMGrapheme32 g;
             ready = MVM_unicode_normalizer_process_codepoint_to_grapheme(tc, &norm, codepoint, &g);
@@ -205,13 +203,8 @@ MVMString * MVM_string_utf8_decode(MVMThreadContext *tc, const MVMObject *result
                     ));
                 }
                 buffer[count++] = g;
-                lowest_graph = g < lowest_graph ? g : lowest_graph;
-                highest_graph = g > highest_graph ? g : highest_graph;
                 while (--ready > 0) {
-                    g = MVM_unicode_normalizer_get_grapheme(tc, &norm);
-                    lowest_graph = g < lowest_graph ? g : lowest_graph;
-                    highest_graph = g > highest_graph ? g : highest_graph;
-                    buffer[count++] = g;
+                    buffer[count++] = MVM_unicode_normalizer_get_grapheme(tc, &norm);
                 }
             }
             break;
@@ -266,20 +259,15 @@ MVMString * MVM_string_utf8_decode(MVMThreadContext *tc, const MVMObject *result
             buffer = MVM_realloc(buffer, sizeof(MVMGrapheme32) * (count + ready));
         }
         while (ready--) {
-            MVMGrapheme32 g;
-            g = MVM_unicode_normalizer_get_grapheme(tc, &norm);
-            lowest_graph = g < lowest_graph ? g : lowest_graph;
-            highest_graph = g > highest_graph ? g : highest_graph;
-            buffer[count++] = g;
+            buffer[count++] =  MVM_unicode_normalizer_get_grapheme(tc, &norm);
         }
     }
     MVM_unicode_normalizer_cleanup(tc, &norm);
 
-    /* If we're lucky, we can fit our string in 8 bits per grapheme.
-     * That happens when our lowest value is bigger than -129 and our
-     * highest value is lower than 128. */
-    if (-128 <= lowest_graph && highest_graph <= 127) {
+    /* If we're lucky, we can fit our string in 8 bits per grapheme. */
+    if (MVM_string_buf32_can_fit_into_8bit(buffer, count)) {
         MVMGrapheme8 *new_buffer = MVM_malloc(sizeof(MVMGrapheme8) * count);
+        MVM_VECTORIZE_LOOP
         for (ready = 0; ready < count; ready++) {
             new_buffer[ready] = buffer[ready];
         }
@@ -382,7 +370,7 @@ MVMuint32 MVM_string_utf8_decodestream(MVMThreadContext *tc, MVMDecodeStream *ds
              * to save on a couple of branches. */
             MVMCodepoint first_significant = ds->norm.first_significant;
             while (lag_codepoint == -1 && pos < cur_bytes->length) {
-                switch(decode_utf8_byte(&state, &codepoint, bytes[pos++])) {
+                switch(MVM_EXPECT(decode_utf8_byte(&state, &codepoint, bytes[pos++]), UTF8_ACCEPT)) {
                 case UTF8_ACCEPT: {
                     if (codepoint == '\r' || codepoint >= first_significant) {
                         can_fast_path = 0;
@@ -403,7 +391,7 @@ MVMuint32 MVM_string_utf8_decodestream(MVMThreadContext *tc, MVMDecodeStream *ds
             }
 
             while (pos < cur_bytes->length) {
-                switch(decode_utf8_byte(&state, &codepoint, bytes[pos++])) {
+                switch(MVM_EXPECT(decode_utf8_byte(&state, &codepoint, bytes[pos++]), UTF8_ACCEPT)) {
                 case UTF8_ACCEPT: {
                     /* If we hit something that needs the normalizer, we put
                      * any lagging codepoint into its buffer and jump to it. */
@@ -465,7 +453,7 @@ MVMuint32 MVM_string_utf8_decodestream(MVMThreadContext *tc, MVMDecodeStream *ds
         }
         else {
             while (pos < cur_bytes->length) {
-                switch(decode_utf8_byte(&state, &codepoint, bytes[pos++])) {
+                switch(MVM_EXPECT(decode_utf8_byte(&state, &codepoint, bytes[pos++]), UTF8_ACCEPT)) {
                 case UTF8_ACCEPT: {
                     MVMGrapheme32 g;
                     MVMint32 first;
