@@ -177,6 +177,10 @@ typedef struct MVMSpeshIterator MVMSpeshIterator;
 typedef struct MVMSpeshLog MVMSpeshLog;
 typedef struct MVMSpeshLogBody MVMSpeshLogBody;
 typedef struct MVMSpeshLogEntry MVMSpeshLogEntry;
+typedef struct MVMSpeshPluginState MVMSpeshPluginState;
+typedef struct MVMSpeshPluginPosition MVMSpeshPluginPosition;
+typedef struct MVMSpeshPluginGuardSet MVMSpeshPluginGuardSet;
+typedef struct MVMSpeshPluginGuard MVMSpeshPluginGuard;
 typedef struct MVMSpeshStats MVMSpeshStats;
 typedef struct MVMSpeshStatsByCallsite MVMSpeshStatsByCallsite;
 typedef struct MVMSpeshStatsByType MVMSpeshStatsByType;
@@ -185,6 +189,7 @@ typedef struct MVMSpeshStatsByOffset MVMSpeshStatsByOffset;
 typedef struct MVMSpeshStatsTypeCount MVMSpeshStatsTypeCount;
 typedef struct MVMSpeshStatsInvokeCount MVMSpeshStatsInvokeCount;
 typedef struct MVMSpeshStatsTypeTupleCount MVMSpeshStatsTypeTupleCount;
+typedef struct MVMSpeshStatsPluginGuardCount MVMSpeshStatsPluginGuardCount;
 typedef struct MVMSpeshStatsStatic MVMSpeshStatsStatic;
 typedef struct MVMSpeshSimStack MVMSpeshSimStack;
 typedef struct MVMSpeshSimStackFrame MVMSpeshSimStackFrame;
