@@ -162,7 +162,7 @@ sub main {
   skip_most:
     break_property('Word', 'Word_Break');
     tweak_nfg_qc();
-
+    find_quick_prop_data();
     # Allocate all the things
     progress("done.\nsetting next_point for codepoints");
     my $first_point = set_next_points();
@@ -201,7 +201,39 @@ sub main {
     print "\nDONE!!!\n\n";
     return 1;
 }
-
+sub find_quick_prop_data {
+    my @wanted_val_str = (
+        [ 'gencat_name', 'Zl', 'Zp' ],
+    );
+    my @wanted_val_bool = (
+        [ 'White_Space', 1 ]
+    );
+    my %gencat_wanted_h;
+    my @result;
+    for my $code (sort { $a <=> $b } keys %{$POINTS_BY_CODE}) {
+        for my $cat_data (@wanted_val_str) {
+            my $propname = $cat_data->[0];
+            my $i;
+            for ($i = 1; $i < @$cat_data; $i++) {
+                my $pval = $cat_data->[$i];
+                push @{$gencat_wanted_h{$propname . "_" . $pval}}, $code if $POINTS_BY_CODE->{$code}->{$propname} eq $pval;
+            }
+        }
+        for my $cat_data (@wanted_val_bool) {
+            my $propname = $cat_data->[0];
+            push @{$gencat_wanted_h{$propname}}, $code if $POINTS_BY_CODE->{$code}->{$propname};
+        }
+    }
+    say Dumper(%gencat_wanted_h);
+    for my $pname (sort keys %gencat_wanted_h) {
+        my @text;
+        for my $cp (@{$gencat_wanted_h{$pname}}) {
+            push @text, "((cp) == $cp)";
+        }
+        push @result, ("#define MVM_CP_is_$pname(cp) (" . join(' || ', @text) . ')');
+    }
+    write_file("src/strings/unicode_prop_macros.h", (join("\n", @result) . "\n"));
+}
 sub thousands {
     my $in = shift;
     $in = reverse "$in"; # stringify or copy the string
@@ -752,17 +784,17 @@ static MVMint32 MVM_codepoint_to_row_index(MVMThreadContext *tc, MVMint64 codepo
 
     MVMint32 plane = codepoint >> 16;
 
-    if (codepoint < 0) {
+    if (MVM_UNLIKELY(codepoint < 0)) {
         MVM_exception_throw_adhoc(tc,
             "Internal Error: MVM_codepoint_to_row_index call requested a synthetic codepoint that does not exist.\n"
             "Requested synthetic %%"PRId64" when only %%"PRId32" have been created.",
             -codepoint, tc->instance->nfg->num_synthetics);
     }
 
-    if (plane == 0) {%s
+    if (MVM_LIKELY(plane == 0)) {%s
     }
     else {
-        if (plane < 0 || plane > 16 || codepoint > 0x10FFFD) {
+        if (MVM_UNLIKELY(plane < 0 || plane > 16 || codepoint > 0x10FFFD)) {
             return -1;
         }
         else {%s
@@ -859,7 +891,6 @@ sub emit_property_value_lookup {
     chomp(my $int_out = <<'END');
 
 static MVMint32 MVM_unicode_get_property_int(MVMThreadContext *tc, MVMint64 codepoint, MVMint64 property_code) {
-    MVMuint32 switch_val = (MVMuint32)property_code;
     MVMint32 result_val = 0; /* we'll never have negatives, but so */
     MVMuint32 codepoint_row = MVM_codepoint_to_row_index(tc, codepoint);
     MVMuint16 bitfield_row;
@@ -868,13 +899,11 @@ static MVMint32 MVM_unicode_get_property_int(MVMThreadContext *tc, MVMint64 code
         /* Unassigned codepoints have General Category Cn. Since this returns 0
          * for unknowns, unless we return 1 for property C then these unknows
          * won't match with <:C> */
-        if (property_code == MVM_UNICODE_PROPERTY_C)
-            return 1;
-        return 0;
+        return property_code == MVM_UNICODE_PROPERTY_C ? 1 : 0;
     }
     bitfield_row = codepoint_bitfield_indexes[codepoint_row];
 
-    switch (switch_val) {
+    switch (MVM_EXPECT(property_code, MVM_UNICODE_PROPERTY_GENERAL_CATEGORY)) {
         case 0: return 0;
 END
 
@@ -1114,6 +1143,7 @@ static void generate_codepoints_by_name(MVMThreadContext *tc) {
         if (codepoint_table_index >= MVM_CODEPOINT_NAMES_COUNT)
             continue;
         switch (codepoint_extents[extent_index][1]) {
+            /* Fate Normal */
             case $FATE_NORMAL: {
                 MVMint32 extent_span_index = 0;
                 codepoint_table_index = codepoint_extents[extent_index][2];
@@ -1131,9 +1161,13 @@ static void generate_codepoints_by_name(MVMThreadContext *tc) {
                 }
                 break;
             }
+            /* Fate NULL */
             case $FATE_NULL:
+                /* codepoint's += ends up being unused since it gets reset at the next loop.
+                 * For now leaving the code as it was. */
                 codepoint += length;
                 break;
+            /* Fate Span */
             case $FATE_SPAN: {
                 const char *name = codepoint_names[codepoint_table_index];
                 if (name) {
@@ -1762,9 +1796,6 @@ sub UnicodeData {
 
         my $code = hex $code_str;
         my $plane_num = $code >> 16;
-        if ($name eq '<control>' ) {
-            $name = sprintf '<control-%.4X>', $code;
-        }
         my $point = get_next_point($code, 1);
         my $hashy = {
             # Unicode_1_Name is not used yet. We should make sure it ends up
@@ -1811,6 +1842,54 @@ sub UnicodeData {
             }
             $ideograph_start = 0;
         }
+        if (substr($point->{name}, 0, 1) eq '<') {
+            if ($point->{name} eq '<CJK Ideograph Extension A>'
+             || $point->{name} eq '<CJK Ideograph>'
+             || $point->{name} eq '<CJK Ideograph Extension B>'
+             || $point->{name} eq '<CJK Ideograph Extension C>'
+             || $point->{name} eq '<CJK Ideograph Extension D>'
+             || $point->{name} eq '<CJK Ideograph Extension E>'
+             || $point->{name} eq '<CJK Ideograph Extension F>')
+            {
+                $point->{name} = '<CJK UNIFIED IDEOGRAPH>'
+            }
+            elsif ($point->{name} eq '<Tangut Ideograph>') {
+                $point->{name} = '<TANGUT IDEOGRAPH>';
+            }
+            elsif ($point->{name} eq '<Hangul Syllable>') {
+                $point->{name} = '<HANGUL SYLLABLE>';
+            }
+            elsif ($point->{name} eq '<Private Use>'
+                || $point->{name} =~ /^<Plane \d+ Private Use>$/)
+            {
+                die unless $gencat eq 'Co';
+                $point->{name} = '<private-use>';
+            }
+            elsif ($point->{name} eq '<Non Private Use High Surrogate>'
+                || $point->{name} eq '<Private Use High Surrogate>'
+                || $point->{name} eq '<Low Surrogate>')
+            {
+                die unless $gencat eq 'Cs';
+                $point->{name} = '<surrogate>';
+            }
+            if ($point->{name} eq '<HANGUL SYLLABLE>'
+             || $point->{name} eq '<control>'
+             || $point->{name} eq '<CJK UNIFIED IDEOGRAPH>'
+             || $point->{name} eq '<private-use>'
+             || $point->{name} eq '<surrogate>'
+             || $point->{name} eq '<TANGUT IDEOGRAPH>') {
+            }
+            else {
+                die "$point->{name} encountered. Make sure to check https://www.unicode.org/versions/Unicode10.0.0/ch04.pdf for Name Derivation Rule Prefix Strings\n" .
+                "Also you will likely have to make a change to MVM_unicode_get_name() and add a test to nqp";
+                say $code_str;
+                exit;
+            }
+        }
+        if ($point->{name} =~ /^CJK COMPATIBILITY IDEOGRAPH-([A-F0-9]+)$/) {
+            die unless sprintf("%.4X", $code) eq $1;
+            $point->{name} = "<CJK COMPATIBILITY IDEOGRAPH>";
+        }
     };
     each_line('UnicodeData', $s);
     $s->("110000;Out of Range;Cn;0;L;;;;;N;;;;;");
@@ -1979,7 +2058,7 @@ sub Jamo {
     });
     my @hangul_syllables;
     for my $key (sort keys %{$POINTS_BY_CODE}) {
-        if ($POINTS_BY_CODE->{$key}->{name} and $POINTS_BY_CODE->{$key}->{name} eq '<Hangul Syllable>') {
+        if ($POINTS_BY_CODE->{$key}->{name} and $POINTS_BY_CODE->{$key}->{name} eq '<HANGUL SYLLABLE>') {
             push @hangul_syllables, $key;
         }
     }
@@ -1988,7 +2067,7 @@ sub Jamo {
     my @out_lines = split "\n", $out;
     my $i = 0;
     for my $line (@out_lines) {
-        my $final_name = 'Hangul Syllable ';
+        my $final_name = 'HANGUL SYLLABLE ';
         my $hs_cps = $hangul_syllables[$i++];
         my @a = split ',', $line;
         for my $cp (@a) {
